@@ -1,7 +1,8 @@
 //! Intermediate Representations of Abstract Syntax Tree
 //!
+pub mod decl;
+pub mod generic;
 pub mod pl;
 pub mod rq;
 
-pub use prql_ast::expr::generic;
-pub use prql_ast::Span;
+pub use prqlc_ast::Span;
