@@ -1,9 +1,9 @@
 use enum_as_inner::EnumAsInner;
-use prql_ast::expr::{Ident, Literal};
+use prqlc_ast::Ty;
 use serde::{Deserialize, Serialize};
 
-use crate::generic::WindowKind;
-use crate::ir::pl::{Expr, ExprKind, Func, FuncCall, Range, Ty};
+use crate::ir::generic::WindowKind;
+use crate::ir::pl::{Expr, ExprKind, Func, FuncCall, Range};
 
 impl FuncCall {
     pub fn new_simple(name: Expr, args: Vec<Expr>) -> Self {
@@ -30,8 +30,8 @@ impl Func {
     }
 }
 
-pub type WindowFrame = crate::generic::WindowFrame<Box<Expr>>;
-pub type ColumnSort = crate::generic::ColumnSort<Box<Expr>>;
+pub type WindowFrame = crate::ir::generic::WindowFrame<Box<Expr>>;
+pub type ColumnSort = crate::ir::generic::ColumnSort<Box<Expr>>;
 
 /// FuncCall with better typing. Returns the modified table.
 #[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
@@ -133,21 +133,3 @@ impl Expr {
         }
     }
 }
-
-impl From<Literal> for ExprKind {
-    fn from(value: Literal) -> Self {
-        ExprKind::Literal(value)
-    }
-}
-
-impl From<Ident> for ExprKind {
-    fn from(value: Ident) -> Self {
-        ExprKind::Ident(value)
-    }
-}
-
-impl From<Func> for ExprKind {
-    fn from(value: Func) -> Self {
-        ExprKind::Func(Box::new(value))
-    }
-}
