@@ -716,6 +716,8 @@ pub const RESERVED_FOR_TABLE_ALIAS: &[Keyword] = &[
     Keyword::QUALIFY,
     Keyword::WINDOW,
     Keyword::END,
+    // for MYSQL PARTITION SELECTION
+    Keyword::PARTITION,
 ];
 
 /// Can't be used as a column alias, so that `SELECT <expr> alias`
