@@ -56,7 +56,7 @@
 //!
 //!     For compiling whole files (`.prql` to `.sql`), call `prql-compiler`
 //!     from `build.rs`.
-//!     See [this example project](https://github.com/PRQL/prql/tree/main/crates/prql_compiler/examples/compile-files).
+//!     See [this example project](https://github.com/PRQL/prql/tree/main/prqlc/prql-compiler/examples/compile-files).
 //!
 //! - Compile, format & debug PRQL from command line.
 //!
@@ -69,8 +69,18 @@
 //!
 //! The following [feature flags](https://doc.rust-lang.org/cargo/reference/manifest.html#the-features-section) are available:
 //!
-//! * `serde_yaml`: adapts the `Serialize` implementation for [`prql_ast::expr::ExprKind::Literal`]
+//! * `serde_yaml`: adapts the `Serialize` implementation for [`prqlc_ast::expr::ExprKind::Literal`]
 //!   to `serde_yaml`, which doesn't support the serialization of nested enums
+//!
+//! ## Large binary sizes
+//!
+//! For Linux users, the binary produced by this crate will probably be quite large (>200MB) by default.
+//! That is because it includes a lot of debuginfo symbols from our parser. You can remove them by
+//! adding the following to `Cargo.toml`:
+//! ```toml
+//! [profile.release.package.prql-compiler]
+//! strip = "debuginfo"
+//! ```
 
 #![forbid(unsafe_code)]
 // Our error type is 128 bytes, because it contains 5 strings & an Enum, which
@@ -81,20 +91,16 @@
 #![allow(clippy::result_large_err)]
 
 mod codegen;
-mod error;
-mod generic;
+mod error_message;
 pub mod ir;
 mod parser;
 pub mod semantic;
 pub mod sql;
-#[cfg(test)]
-mod tests;
 mod utils;
 
-pub use error::{
-    downcast, Error, ErrorMessage, ErrorMessages, MessageKind, Reason, SourceLocation,
-};
+pub use error_message::{downcast, ErrorMessage, ErrorMessages, SourceLocation, WithErrorInfo};
 pub use ir::Span;
+pub use prqlc_ast::error::{Error, Errors, MessageKind, Reason};
 
 use once_cell::sync::Lazy;
 use semver::Version;
@@ -139,7 +145,7 @@ pub fn compile(prql: &str, options: &Options) -> Result<String, ErrorMessages> {
     parser::parse(&sources)
         .and_then(|ast| semantic::resolve_and_lower(ast, &[]))
         .and_then(|rq| sql::compile(rq, options))
-        .map_err(error::downcast)
+        .map_err(error_message::downcast)
         .map_err(|e| e.composed(&prql.into()))
 }
 
@@ -263,46 +269,46 @@ impl Options {
 pub struct ReadmeDoctests;
 
 /// Parse PRQL into a PL AST
-pub fn prql_to_pl(prql: &str) -> Result<Vec<prql_ast::stmt::Stmt>, ErrorMessages> {
+pub fn prql_to_pl(prql: &str) -> Result<Vec<prqlc_ast::stmt::Stmt>, ErrorMessages> {
     let sources = SourceTree::from(prql);
 
     parser::parse(&sources)
         .map(|x| x.sources.into_values().next().unwrap())
-        .map_err(error::downcast)
+        .map_err(error_message::downcast)
         .map_err(|e| e.composed(&prql.into()))
 }
 
 /// Parse PRQL into a PL AST
 pub fn prql_to_pl_tree(
     prql: &SourceTree,
-) -> Result<SourceTree<Vec<prql_ast::stmt::Stmt>>, ErrorMessages> {
+) -> Result<SourceTree<Vec<prqlc_ast::stmt::Stmt>>, ErrorMessages> {
     parser::parse(prql)
-        .map_err(error::downcast)
+        .map_err(error_message::downcast)
         .map_err(|e| e.composed(prql))
 }
 
 /// Perform semantic analysis and convert PL to RQ.
-pub fn pl_to_rq(pl: Vec<prql_ast::stmt::Stmt>) -> Result<ir::rq::RelationalQuery, ErrorMessages> {
+pub fn pl_to_rq(pl: Vec<prqlc_ast::stmt::Stmt>) -> Result<ir::rq::RelationalQuery, ErrorMessages> {
     let source_tree = SourceTree::single(PathBuf::new(), pl);
-    semantic::resolve_and_lower(source_tree, &[]).map_err(error::downcast)
+    semantic::resolve_and_lower(source_tree, &[]).map_err(error_message::downcast)
 }
 
 /// Perform semantic analysis and convert PL to RQ.
 pub fn pl_to_rq_tree(
-    pl: SourceTree<Vec<prql_ast::stmt::Stmt>>,
+    pl: SourceTree<Vec<prqlc_ast::stmt::Stmt>>,
     main_path: &[String],
 ) -> Result<ir::rq::RelationalQuery, ErrorMessages> {
-    semantic::resolve_and_lower(pl, main_path).map_err(error::downcast)
+    semantic::resolve_and_lower(pl, main_path).map_err(error_message::downcast)
 }
 
 /// Generate SQL from RQ.
 pub fn rq_to_sql(rq: ir::rq::RelationalQuery, options: &Options) -> Result<String, ErrorMessages> {
-    sql::compile(rq, options).map_err(error::downcast)
+    sql::compile(rq, options).map_err(error_message::downcast)
 }
 
 /// Generate PRQL code from PL AST
-pub fn pl_to_prql(pl: Vec<prql_ast::stmt::Stmt>) -> Result<String, ErrorMessages> {
-    Ok(codegen::write_stmts(&pl))
+pub fn pl_to_prql(pl: Vec<prqlc_ast::stmt::Stmt>) -> Result<String, ErrorMessages> {
+    Ok(codegen::WriteSource::write(&pl, codegen::WriteOpt::default()).unwrap())
 }
 
 /// JSON serialization and deserialization functions
@@ -310,12 +316,12 @@ pub mod json {
     use super::*;
 
     /// JSON serialization
-    pub fn from_pl(pl: Vec<prql_ast::stmt::Stmt>) -> Result<String, ErrorMessages> {
+    pub fn from_pl(pl: Vec<prqlc_ast::stmt::Stmt>) -> Result<String, ErrorMessages> {
         serde_json::to_string(&pl).map_err(|e| anyhow::anyhow!(e).into())
     }
 
     /// JSON deserialization
-    pub fn to_pl(json: &str) -> Result<Vec<prql_ast::stmt::Stmt>, ErrorMessages> {
+    pub fn to_pl(json: &str) -> Result<Vec<prqlc_ast::stmt::Stmt>, ErrorMessages> {
         serde_json::from_str(json).map_err(|e| anyhow::anyhow!(e).into())
     }
 
@@ -379,11 +385,34 @@ impl<S: ToString> From<S> for SourceTree {
 }
 
 #[cfg(test)]
-mod tests_lib {
+mod tests {
     use crate::Target;
     use insta::assert_debug_snapshot;
+    use prqlc_ast::expr::Ident;
     use std::str::FromStr;
 
+    pub fn compile(prql: &str) -> Result<String, super::ErrorMessages> {
+        anstream::ColorChoice::Never.write_global();
+        super::compile(prql, &super::Options::default().no_signature())
+    }
+
+    #[test]
+    fn test_starts_with() {
+        // Over-testing, from co-pilot, can remove some of them.
+        let a = Ident::from_path(vec!["a", "b", "c"]);
+        let b = Ident::from_path(vec!["a", "b"]);
+        let c = Ident::from_path(vec!["a", "b", "c", "d"]);
+        let d = Ident::from_path(vec!["a", "b", "d"]);
+        let e = Ident::from_path(vec!["a", "c"]);
+        let f = Ident::from_path(vec!["b", "c"]);
+        assert!(a.starts_with(&b));
+        assert!(a.starts_with(&a));
+        assert!(!a.starts_with(&c));
+        assert!(!a.starts_with(&d));
+        assert!(!a.starts_with(&e));
+        assert!(!a.starts_with(&f));
+    }
+
     #[test]
     fn test_target_from_str() {
         assert_debug_snapshot!(Target::from_str("sql.postgres"), @r###"
