@@ -2,23 +2,24 @@ use std::collections::{HashMap, HashSet};
 
 use anyhow::{anyhow, bail, Result};
 use itertools::Itertools;
+use prqlc_ast::TupleField;
 use serde::Deserialize;
 use std::iter::zip;
 
-use crate::error::{Error, Reason, WithErrorInfo};
-use crate::generic::{SortDirection, WindowKind};
+use prqlc_ast::error::{Error, Reason};
+
+use crate::ir::decl::{Decl, DeclKind, Module, RootModule};
+use crate::ir::generic::{SortDirection, WindowKind};
 use crate::ir::pl::PlFold;
 use crate::ir::pl::*;
-use crate::semantic::write_pl;
+use crate::semantic::ast_expand::try_restrict_range;
+use crate::semantic::{write_pl, NS_PARAM, NS_THIS};
+use crate::{WithErrorInfo, COMPILER_VERSION};
 
-use super::super::decl::{Decl, DeclKind};
-use super::super::module::Module;
 use super::Resolver;
-use super::{Lineage, RootModule};
-use super::{NS_PARAM, NS_THIS};
 
 /// try to convert function call with enough args into transform
-pub fn cast_transform(resolver: &mut Resolver, closure: Func) -> Result<Expr> {
+pub fn resolve_special_func(resolver: &mut Resolver, closure: Func) -> Result<Expr> {
     let internal_name = closure.body.kind.as_internal().unwrap();
 
     let (kind, input) = match internal_name.as_str() {
@@ -74,19 +75,22 @@ pub fn cast_transform(resolver: &mut Resolver, closure: Func) -> Result<Expr> {
         "take" => {
             let [expr, tbl] = unpack::<2>(closure);
 
-            let range = match expr.kind {
-                ExprKind::Literal(Literal::Integer(n)) => range_from_ints(None, Some(n)),
-                ExprKind::Range(range) => range,
-                _ => {
-                    return Err(Error::new(Reason::Expected {
-                        who: Some("`take`".to_string()),
-                        expected: "int or range".to_string(),
-                        found: write_pl(expr.clone()),
-                    })
-                    // Possibly this should refer to the item after the `take` where
-                    // one exists?
-                    .with_span(expr.span)
-                    .into());
+            let range = if let ExprKind::Literal(Literal::Integer(n)) = expr.kind {
+                range_from_ints(None, Some(n))
+            } else {
+                match try_restrict_range(expr) {
+                    Ok(range) => range,
+                    Err(expr) => {
+                        return Err(Error::new(Reason::Expected {
+                            who: Some("`take`".to_string()),
+                            expected: "int or range".to_string(),
+                            found: write_pl(expr.clone()),
+                        })
+                        // Possibly this should refer to the item after the `take` where
+                        // one exists?
+                        .with_span(expr.span)
+                        .into());
+                    }
                 }
             };
 
@@ -157,9 +161,21 @@ pub fn cast_transform(resolver: &mut Resolver, closure: Func) -> Result<Expr> {
                 })?
             };
 
-            let rows = rows.try_cast(|r| r.into_range(), Some("parameter `rows`"), "a range")?;
+            let rows = try_restrict_range(rows).map_err(|e| {
+                Error::new(Reason::Expected {
+                    who: Some("parameter `rows`".to_string()),
+                    expected: "a range".to_string(),
+                    found: write_pl(e),
+                })
+            })?;
 
-            let range = range.try_cast(|r| r.into_range(), Some("parameter `range`"), "a range")?;
+            let range = try_restrict_range(range).map_err(|e| {
+                Error::new(Reason::Expected {
+                    who: Some("parameter `range`".to_string()),
+                    expected: "a range".to_string(),
+                    found: write_pl(e),
+                })
+            })?;
 
             let (kind, range) = if expanding {
                 (WindowKind::Rows, range_from_ints(None, Some(0)))
@@ -205,8 +221,8 @@ pub fn cast_transform(resolver: &mut Resolver, closure: Func) -> Result<Expr> {
 
             let [pattern, value] = unpack::<2>(closure);
 
-            match pattern.kind {
-                ExprKind::Range(Range { start, end }) => {
+            let pattern = match try_restrict_range(pattern) {
+                Ok(Range { start, end }) => {
                     let start = start.map(|s| new_binop(value.clone(), &["std", "gte"], *s));
                     let end = end.map(|end| new_binop(value, &["std", "lte"], *end));
 
@@ -215,13 +231,9 @@ pub fn cast_transform(resolver: &mut Resolver, closure: Func) -> Result<Expr> {
                         res.unwrap_or_else(|| Expr::new(ExprKind::Literal(Literal::Boolean(true))));
                     return Ok(res);
                 }
-                ExprKind::Tuple(_) => {
-                    // TODO: should translate into `value IN (...)`
-                    //   but RQ currently does not support sub queries or
-                    //   even expressions that evaluate to a tuple.
-                }
-                _ => {}
-            }
+                Err(expr) => expr,
+            };
+
             return Err(Error::new(Reason::Expected {
                 who: Some("std.in".to_string()),
                 expected: "a pattern".to_string(),
@@ -366,6 +378,12 @@ pub fn cast_transform(resolver: &mut Resolver, closure: Func) -> Result<Expr> {
             return Ok(res);
         }
 
+        "prql_version" => {
+            // yes, this is not a transform, but this is the most appropriate place for it
+            let ver = COMPILER_VERSION.to_string();
+            return Ok(Expr::new(ExprKind::Literal(Literal::String(ver))));
+        }
+
         _ => {
             return Err(Error::new_simple("unknown operator {internal_name}")
                 .push_hint("this is a bug in prql-compiler")
@@ -466,11 +484,11 @@ fn fold_by_simulating_eval(
     )));
 
     let env = Module::singleton(param_name, Decl::from(DeclKind::Column(param_id)));
-    resolver.context.root_mod.stack_push(NS_PARAM, env);
+    resolver.root_mod.module.stack_push(NS_PARAM, env);
 
     let pipeline = resolver.fold_expr(pipeline)?;
 
-    resolver.context.root_mod.stack_pop(NS_PARAM).unwrap();
+    resolver.root_mod.module.stack_pop(NS_PARAM).unwrap();
 
     // now, we need wrap the result into a closure and replace
     // the dummy node with closure's parameter.
@@ -498,7 +516,7 @@ fn fold_by_simulating_eval(
 }
 
 impl TransformCall {
-    pub fn infer_type(&self, context: &RootModule) -> Result<Lineage> {
+    pub fn infer_type(&self, root_mod: &RootModule) -> Result<Lineage> {
         use TransformKind::*;
 
         fn ty_frame_or_default(expr: &Expr) -> Result<Lineage> {
@@ -512,13 +530,13 @@ impl TransformCall {
                 let mut frame = ty_frame_or_default(&self.input)?;
 
                 frame.clear();
-                frame.apply_assigns(assigns, context);
+                frame.apply_assigns(assigns, root_mod);
                 frame
             }
             Derive { assigns } => {
                 let mut frame = ty_frame_or_default(&self.input)?;
 
-                frame.apply_assigns(assigns, context);
+                frame.apply_assigns(assigns, root_mod);
                 frame
             }
             Group { pipeline, by, .. } => {
@@ -540,7 +558,7 @@ impl TransformCall {
                         frame.columns = Vec::new();
 
                         log::debug!(".. group by {by:?}");
-                        frame.apply_assigns(by, context);
+                        frame.apply_assigns(by, root_mod);
 
                         frame.columns.extend(aggregate_columns);
                     }
@@ -562,7 +580,7 @@ impl TransformCall {
                 let mut frame = ty_frame_or_default(&self.input)?;
                 frame.clear();
 
-                frame.apply_assigns(assigns, context);
+                frame.apply_assigns(assigns, root_mod);
                 frame
             }
             Join { with, .. } => {
@@ -603,8 +621,8 @@ fn append(mut top: Lineage, bottom: Lineage) -> Result<Lineage, Error> {
     let mut columns = Vec::with_capacity(top.columns.len());
     for (t, b) in zip(top.columns, bottom.columns) {
         columns.push(match (t, b) {
-            (LineageColumn::All { input_name, except }, LineageColumn::All { .. }) => {
-                LineageColumn::All { input_name, except }
+            (LineageColumn::All { input_id, except }, LineageColumn::All { .. }) => {
+                LineageColumn::All { input_id, except }
             }
             (
                 LineageColumn::Single {
@@ -650,7 +668,7 @@ impl Lineage {
         self.prev_columns.append(&mut self.columns);
     }
 
-    pub fn apply_assign(&mut self, expr: &Expr, context: &RootModule) {
+    pub fn apply_assign(&mut self, expr: &Expr, root_mod: &RootModule) {
         // spacial case: all except
         if let ExprKind::All { except, .. } = &expr.kind {
             let except_exprs: HashSet<&usize> =
@@ -666,7 +684,7 @@ impl Lineage {
                         if except_inputs.contains(target_id) {
                             continue;
                         }
-                        self.columns.extend(input.get_all_columns(except, context));
+                        self.columns.extend(input.get_all_columns(except, root_mod));
                     }
                     None => {
                         // include the column with if target_id
@@ -712,16 +730,20 @@ impl Lineage {
         });
     }
 
-    pub fn apply_assigns(&mut self, assigns: &[Expr], context: &RootModule) {
+    pub fn apply_assigns(&mut self, assigns: &[Expr], root_mod: &RootModule) {
         for expr in assigns {
-            self.apply_assign(expr, context);
+            self.apply_assign(expr, root_mod);
         }
     }
 
-    pub fn find_input(&self, input_name: &str) -> Option<&LineageInput> {
+    pub fn find_input_by_name(&self, input_name: &str) -> Option<&LineageInput> {
         self.inputs.iter().find(|i| i.name == input_name)
     }
 
+    pub fn find_input(&self, input_id: usize) -> Option<&LineageInput> {
+        self.inputs.iter().find(|i| i.id == input_id)
+    }
+
     /// Renames all frame inputs to given alias.
     pub fn rename(&mut self, alias: String) {
         for input in &mut self.inputs {
@@ -730,7 +752,7 @@ impl Lineage {
 
         for col in &mut self.columns {
             match col {
-                LineageColumn::All { input_name, .. } => *input_name = alias.clone(),
+                LineageColumn::All { .. } => {}
                 LineageColumn::Single {
                     name: Some(name), ..
                 } => name.path = vec![alias.clone()],
@@ -741,8 +763,8 @@ impl Lineage {
 }
 
 impl LineageInput {
-    fn get_all_columns(&self, except: &[Expr], context: &RootModule) -> Vec<LineageColumn> {
-        let rel_def = context.root_mod.get(&self.table).unwrap();
+    fn get_all_columns(&self, except: &[Expr], root_mod: &RootModule) -> Vec<LineageColumn> {
+        let rel_def = root_mod.module.get(&self.table).unwrap();
         let rel_def = rel_def.kind.as_table_decl().unwrap();
 
         // TODO: can this panic?
@@ -770,7 +792,7 @@ impl LineageInput {
                 .collect();
 
             return vec![LineageColumn::All {
-                input_name: self.name.clone(),
+                input_id: self.id,
                 except,
             }];
         }
