@@ -9,13 +9,13 @@ use sqlparser::ast::{
 };
 use std::cmp::Ordering;
 
-use crate::error::{Error, Span, WithErrorInfo};
-use crate::generic::{ColumnSort, SortDirection, WindowFrame, WindowKind};
-use crate::ir::generic::{InterpolateItem, Range};
+use crate::ir::generic::{ColumnSort, SortDirection, WindowFrame, WindowKind};
 use crate::ir::pl::{self, Ident, Literal};
 use crate::ir::rq::*;
 use crate::sql::srq::context::ColumnDecl;
 use crate::utils::{OrMap, VALID_IDENT};
+use crate::{Error, Span, WithErrorInfo};
+use prqlc_ast::expr::generic::{InterpolateItem, Range};
 
 use super::gen_projection::try_into_exprs;
 use super::{keywords, Context};
@@ -395,9 +395,10 @@ pub(super) fn translate_cid(cid: CId, ctx: &mut Context) -> Result<ExprOrSource>
                 let window = compute.window.clone();
                 let span = compute.expr.span;
 
+                let prev_wf = ctx.query.window_function;
                 ctx.query.window_function = window.is_some();
                 let expr = translate_expr(compute.expr.clone(), ctx)?;
-                ctx.query.window_function = false;
+                ctx.query.window_function = prev_wf;
 
                 if let Some(window) = window {
                     translate_windowed(expr, window, ctx, span)?
@@ -875,6 +876,8 @@ pub enum ExprOrSource {
 pub struct SourceExpr {
     pub text: String,
     pub binding_strength: i32,
+
+    /// True for expressions that support (and need) the window frame (OVER ...)
     pub window_frame: bool,
 }
 
@@ -933,8 +936,8 @@ impl From<sql_ast::Expr> for ExprOrSource {
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::ir::generic::Range;
     use insta::assert_yaml_snapshot;
+    use prqlc_ast::expr::generic::Range;
 
     #[test]
     fn test_range_of_ranges() -> Result<()> {
