@@ -10,12 +10,12 @@ use sqlparser::ast::{
     TableFactor, TableWithJoins,
 };
 
-use crate::error::WithErrorInfo;
-use crate::ir::generic::InterpolateItem;
 use crate::ir::pl::{JoinSide, Literal};
 use crate::ir::rq::{CId, Expr, ExprKind, RelationLiteral, RelationalQuery};
 use crate::utils::{BreakUp, Pluck};
 use crate::Error;
+use crate::WithErrorInfo;
+use prqlc_ast::expr::generic::InterpolateItem;
 
 use super::gen_expr::*;
 use super::gen_projection::*;
@@ -119,24 +119,14 @@ fn translate_select_pipeline(
     let distinct = if is_distinct {
         Some(sql_ast::Distinct::Distinct)
     } else if !distinct_ons.is_empty() {
-        // FIXME: this "works" but is very hacky — it's using the
-        // `translate_select_item` to translate each id into an Expr — is that
-        // correct?
-        //
-        // Do we know we won't have more than one DistinctOn? We have that for
-        // `Distinct`. But this will panic if we have more than one.
-        // And if there the `SelectItem` is not an `UnnamedExpr`, it'll panic.
         Some(sql_ast::Distinct::On(
             distinct_ons
                 .into_iter()
-                .exactly_one()?
+                .exactly_one()
+                .unwrap()
                 .into_iter()
-                .map(|id| translate_select_item(id, ctx).unwrap())
-                .map(|item| match item {
-                    SelectItem::UnnamedExpr(expr) => expr,
-                    _ => unreachable!(),
-                })
-                .collect::<Vec<sql_ast::Expr>>(),
+                .map(|id| translate_cid(id, ctx).map(|x| x.into_ast()))
+                .collect::<Result<Vec<_>>>()?,
         ))
     } else {
         None
@@ -154,7 +144,7 @@ fn translate_select_pipeline(
     let aggregate = after_agg.pluck(|t| t.into_aggregate()).into_iter().next();
     let group_by: Vec<CId> = aggregate.map(|(part, _)| part).unwrap_or_default();
     ctx.query.allow_stars = ctx.dialect.stars_in_group();
-    let group_by = try_into_exprs(group_by, ctx, None)?;
+    let group_by = sql_ast::GroupByExpr::Expressions(try_into_exprs(group_by, ctx, None)?);
     ctx.query.allow_stars = true;
 
     ctx.query.pre_projection = false;
@@ -276,7 +266,7 @@ fn translate_relation_expr(relation_expr: RelationExpr, ctx: &mut Context) -> Re
 
     Ok(match relation_expr.kind {
         RelationExprKind::Ref(tid) => {
-            let decl = ctx.anchor.table_decls.get_mut(&tid).unwrap();
+            let decl = ctx.anchor.lookup_table_decl(&tid).unwrap();
 
             // prepare names
             let table_name = decl.name.clone().unwrap();
@@ -293,6 +283,7 @@ fn translate_relation_expr(relation_expr: RelationExpr, ctx: &mut Context) -> Re
                 args: None,
                 with_hints: vec![],
                 version: None,
+                partitions: vec![],
             }
         }
         RelationExprKind::SubQuery(query) => {
@@ -335,7 +326,7 @@ fn translate_join(
 }
 
 fn translate_cte(cte: Cte, ctx: &mut Context) -> Result<(sql_ast::Cte, bool)> {
-    let decl = ctx.anchor.table_decls.get_mut(&cte.tid).unwrap();
+    let decl = ctx.anchor.lookup_table_decl(&cte.tid).unwrap();
     let cte_name = decl.name.clone().unwrap();
 
     let cte_name = translate_ident(Some(cte_name), None, ctx).pop().unwrap();
@@ -534,7 +525,7 @@ fn default_select() -> Select {
         from: Vec::new(),
         lateral_views: Vec::new(),
         selection: None,
-        group_by: Vec::new(),
+        group_by: sql_ast::GroupByExpr::Expressions(vec![]),
         cluster_by: Vec::new(),
         distribute_by: Vec::new(),
         sort_by: Vec::new(),
